dnl
dnl  libflame
dnl  An object-based infrastructure for developing high-performance
dnl  dense linear algebra libraries.
dnl
dnl  Copyright (C) 2008, The University of Texas
dnl
dnl  libflame is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU Lesser General Public License as
dnl  published by the Free Software Foundation; either version 2.1 of
dnl  the License, or (at your option) any later version.
dnl
dnl  libflame is distributed in the hope that it will be useful, but
dnl  WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
dnl  Lesser General Public License for more details.
dnl
dnl  You should have received a copy of the GNU Lesser General Public
dnl  License along with libflame; if you did not receive a copy, see
dnl  http://www.gnu.org/licenses/.
dnl
dnl  For more information, please contact us at flame@cs.utexas.edu or
dnl  send mail to:
dnl
dnl  Field G. Van Zee and/or
dnl  Robert A. van de Geijn
dnl  The University of Texas at Austin
dnl  Department of Computer Sciences
dnl  1 University Station C0500
dnl  Austin TX 78712
dnl
AC_DEFUN([FLA_OBSERVE_F77_NAME_MANGLING],
[
	dnl This is important. We have to invoke the AC_F77_WRAPPERS macro first so
	dnl it will invoke the name-mangling macro and assign the shell variable
	dnl ac_cv_f77_mangling with an appropriate value.
	AC_REQUIRE([AC_F77_WRAPPERS])
	
	fla_f77_underscore=''
	fla_f77_extra_underscore=''
	
	dnl Now let's inspect the resulting value from the name-mangling check.

	dnl First we look at whether an underscore is appended to symbol names.
	case "$ac_cv_f77_mangling" in

		dnl No underscore is appended.
		*", no underscore,"*)
			fla_f77_underscore=''
		;;
		dnl An underscore is appended.
		*", underscore,"*)
			fla_f77_underscore='_'
		;;
		dnl Unknown underscoring scheme!
		*)
			AC_MSG_ERROR([determined unexpected underscoring scheme!])
		;;
	esac
	
	dnl Then we look at whether the compiler appends an extra underscore to
	dnl symbol names already containing at least one underscore.
	case "$ac_cv_f77_mangling" in

		dnl No additional underscore is appended
		*", no extra underscore")
			fla_f77_extra_underscore=''
		;;
		dnl An additional underscore is appended.
		*", extra underscore")
			fla_f77_extra_underscore='_'
		;;
		dnl Unknown (extra) underscoring scheme!
		*)
			AC_MSG_ERROR([determined unexpected (extra) underscoring scheme!])
		;;
	esac
	
	dnl Handle the results of the test.
	if test "$fla_f77_underscore" = "_" ; then
		AC_DEFINE(FLA_F77_UNDERSCORE,1,
                    [Determines whether the Fortran 77 compiler appends an underscore to symbol names.])
	fi
	if test "$fla_f77_extra_underscore" = "_" ; then
		AC_DEFINE(FLA_F77_EXTRA_UNDERSCORE,1,
                    [Determines whether the Fortran 77 compiler appends an extra underscore to symbol names that already contain at least one underscore.])
	fi
	
	dnl Substitute the output variable.
	fla_f77_name_mangling=$ac_cv_f77_mangling
	AC_SUBST(fla_f77_name_mangling)
	
])
