/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

fla_gemm_t* FLA_Cntl_gemm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_gemm_t* sub_gemm )
{
	fla_gemm_t* cntl;
	
	cntl = ( fla_gemm_t* ) FLA_malloc( sizeof(fla_gemm_t) );
	
	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_gemm    = sub_gemm;

	return cntl;
}

fla_hemm_t* FLA_Cntl_hemm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_hemm_t* sub_hemm, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 )
{
	fla_hemm_t* cntl;
	
	cntl = ( fla_hemm_t* ) FLA_malloc( sizeof(fla_hemm_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_hemm    = sub_hemm;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;

	return cntl;
}

fla_symm_t* FLA_Cntl_symm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_symm_t* sub_symm, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 )
{
	fla_symm_t* cntl;
	
	cntl = ( fla_symm_t* ) FLA_malloc( sizeof(fla_symm_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_symm    = sub_symm;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;

	return cntl;
}

fla_herk_t* FLA_Cntl_herk_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_herk_t* sub_herk, fla_gemm_t* sub_gemm )
{
	fla_herk_t* cntl;
	
	cntl = ( fla_herk_t* ) FLA_malloc( sizeof(fla_herk_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_herk    = sub_herk;
	cntl->sub_gemm    = sub_gemm;

	return cntl;
}

fla_her2k_t* FLA_Cntl_her2k_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_her2k_t* sub_her2k, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 )
{
	fla_her2k_t* cntl;
	
	cntl = ( fla_her2k_t* ) FLA_malloc( sizeof(fla_her2k_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_her2k   = sub_her2k;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;

	return cntl;
}

fla_syrk_t* FLA_Cntl_syrk_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_syrk_t* sub_syrk, fla_gemm_t* sub_gemm )
{
	fla_syrk_t* cntl;
	
	cntl = ( fla_syrk_t* ) FLA_malloc( sizeof(fla_syrk_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_syrk    = sub_syrk;
	cntl->sub_gemm    = sub_gemm;

	return cntl;
}

fla_syr2k_t* FLA_Cntl_syr2k_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_syr2k_t* sub_syr2k, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 )
{
	fla_syr2k_t* cntl;
	
	cntl = ( fla_syr2k_t* ) FLA_malloc( sizeof(fla_syr2k_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_syr2k   = sub_syr2k;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;

	return cntl;
}

fla_trmm_t* FLA_Cntl_trmm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_trmm_t* sub_trmm, fla_gemm_t* sub_gemm )
{
	fla_trmm_t* cntl;
	
	cntl = ( fla_trmm_t* ) FLA_malloc( sizeof(fla_trmm_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_trmm    = sub_trmm;
	cntl->sub_gemm    = sub_gemm;

	return cntl;
}

fla_trsm_t* FLA_Cntl_trsm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_trsm_t* sub_trsm, fla_gemm_t* sub_gemm )
{
	fla_trsm_t* cntl;
	
	cntl = ( fla_trsm_t* ) FLA_malloc( sizeof(fla_trsm_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_trsm    = sub_trsm;
	cntl->sub_gemm    = sub_gemm;

	return cntl;
}

fla_chol_t* FLA_Cntl_chol_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_chol_t* sub_chol, fla_syrk_t* sub_syrk, fla_herk_t* sub_herk, fla_trsm_t* sub_trsm, fla_gemm_t* sub_gemm )
{
	fla_chol_t* cntl;
	
	cntl = ( fla_chol_t* ) FLA_malloc( sizeof(fla_chol_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_chol    = sub_chol;
	cntl->sub_syrk    = sub_syrk;
	cntl->sub_herk    = sub_herk;
	cntl->sub_trsm    = sub_trsm;
	cntl->sub_gemm    = sub_gemm;

	return cntl;
}

fla_lu_t*    FLA_Cntl_lu_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_lu_t* sub_lu, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2, fla_gemm_t* sub_gemm3, fla_trsm_t* sub_trsm1, fla_trsm_t* sub_trsm2 )
{
	fla_lu_t* cntl;
	
	cntl = ( fla_lu_t* ) FLA_malloc( sizeof(fla_lu_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_lu      = sub_lu;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;
	cntl->sub_gemm3   = sub_gemm3;
	cntl->sub_trsm1   = sub_trsm1;
	cntl->sub_trsm2   = sub_trsm2;

	return cntl;
}

fla_qr_t*    FLA_Cntl_qr_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_qr_t* sub_qr, fla_trmm_t* sub_trmm1, fla_trmm_t* sub_trmm2, fla_trmm_t* sub_trmm3, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 )
{
	fla_qr_t* cntl;
	
	cntl = ( fla_qr_t* ) FLA_malloc( sizeof(fla_qr_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_qr      = sub_qr;
	cntl->sub_trmm1   = sub_trmm1;
	cntl->sub_trmm2   = sub_trmm2;
	cntl->sub_trmm3   = sub_trmm3;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;

	return cntl;
}

fla_lq_t*    FLA_Cntl_lq_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_lq_t* sub_lq, fla_trmm_t* sub_trmm1, fla_trmm_t* sub_trmm2, fla_trmm_t* sub_trmm3, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2 )
{
	fla_lq_t* cntl;
	
	cntl = ( fla_lq_t* ) FLA_malloc( sizeof(fla_lq_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_lq      = sub_lq;
	cntl->sub_trmm1   = sub_trmm1;
	cntl->sub_trmm2   = sub_trmm2;
	cntl->sub_trmm3   = sub_trmm3;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;

	return cntl;
}

fla_trinv_t* FLA_Cntl_trinv_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_trinv_t* sub_trinv, fla_trmm_t* sub_trmm, fla_trsm_t* sub_trsm1, fla_trsm_t* sub_trsm2, fla_gemm_t* sub_gemm )
{
	fla_trinv_t* cntl;
	
	cntl = ( fla_trinv_t* ) FLA_malloc( sizeof(fla_trinv_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_trinv   = sub_trinv;
	cntl->sub_trmm    = sub_trmm;
	cntl->sub_trsm1   = sub_trsm1;
	cntl->sub_trsm2   = sub_trsm2;
	cntl->sub_gemm    = sub_gemm;

	return cntl;
}

fla_hess_t*  FLA_Cntl_hess_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_hess_t* sub_hess, fla_trmm_t* sub_trmm1, fla_trmm_t* sub_trmm2, fla_trmm_t* sub_trmm3, fla_trmm_t* sub_trmm4, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2, fla_gemm_t* sub_gemm3 )
{
	fla_hess_t* cntl;
	
	cntl = ( fla_hess_t* ) FLA_malloc( sizeof(fla_hess_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_hess    = sub_hess;
	cntl->sub_trmm1   = sub_trmm1;
	cntl->sub_trmm2   = sub_trmm2;
	cntl->sub_trmm3   = sub_trmm3;
	cntl->sub_trmm4   = sub_trmm4;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;
	cntl->sub_gemm3   = sub_gemm3;

	return cntl;
}

fla_ttmm_t*  FLA_Cntl_ttmm_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_ttmm_t* sub_ttmm, fla_syrk_t* sub_syrk, fla_herk_t* sub_herk, fla_trmm_t* sub_trmm, fla_gemm_t* sub_gemm )
{
	fla_ttmm_t* cntl;
	
	cntl = ( fla_ttmm_t* ) FLA_malloc( sizeof(fla_ttmm_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_ttmm    = sub_ttmm;
	cntl->sub_syrk    = sub_syrk;
	cntl->sub_herk    = sub_herk;
	cntl->sub_trmm    = sub_trmm;
	cntl->sub_gemm    = sub_gemm;

	return cntl;
}

fla_sylv_t*  FLA_Cntl_sylv_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_sylv_t* sub_sylv1, fla_sylv_t* sub_sylv2, fla_sylv_t* sub_sylv3, fla_gemm_t* sub_gemm1, fla_gemm_t* sub_gemm2, fla_gemm_t* sub_gemm3, fla_gemm_t* sub_gemm4, fla_gemm_t* sub_gemm5, fla_gemm_t* sub_gemm6, fla_gemm_t* sub_gemm7, fla_gemm_t* sub_gemm8 )
{
	fla_sylv_t* cntl;
	
	cntl = ( fla_sylv_t* ) FLA_malloc( sizeof(fla_sylv_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_sylv1   = sub_sylv1;
	cntl->sub_sylv2   = sub_sylv2;
	cntl->sub_sylv3   = sub_sylv3;
	cntl->sub_gemm1   = sub_gemm1;
	cntl->sub_gemm2   = sub_gemm2;
	cntl->sub_gemm3   = sub_gemm3;
	cntl->sub_gemm4   = sub_gemm4;
	cntl->sub_gemm5   = sub_gemm5;
	cntl->sub_gemm6   = sub_gemm6;
	cntl->sub_gemm7   = sub_gemm7;
	cntl->sub_gemm8   = sub_gemm8;

	return cntl;
}

fla_transpose_t*  FLA_Cntl_transpose_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_transpose_t* sub_trans, fla_swap_t* sub_swap )
{
	fla_transpose_t* cntl;
	
	cntl = ( fla_transpose_t* ) FLA_malloc( sizeof(fla_transpose_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_trans   = sub_trans;
	cntl->sub_swap    = sub_swap;

	return cntl;
}

fla_swap_t*  FLA_Cntl_swap_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_swap_t* sub_swap )
{
	fla_swap_t* cntl;
	
	cntl = ( fla_swap_t* ) FLA_malloc( sizeof(fla_swap_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_swap    = sub_swap;

	return cntl;
}

fla_spdinv_t* FLA_Cntl_spdinv_obj_create( FLA_Matrix_type matrix_type, int variant, fla_blocksize_t* blocksize, fla_chol_t* sub_chol, fla_trinv_t* sub_trinv, fla_ttmm_t* sub_ttmm )
{
	fla_spdinv_t* cntl;
	
	cntl = ( fla_spdinv_t* ) FLA_malloc( sizeof(fla_spdinv_t) );

	cntl->matrix_type = matrix_type;
	cntl->variant     = variant;
	cntl->blocksize   = blocksize;
	cntl->sub_chol    = sub_chol;
	cntl->sub_trinv   = sub_trinv;
	cntl->sub_ttmm    = sub_ttmm;

	return cntl;
}

void FLA_Cntl_obj_free( void* cntl )
{
	FLA_free( cntl );
}
