/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

extern fla_gemm_t* fla_gemm_cntl_blas;
extern fla_trmm_t* fla_trmm_cntl_blas;

fla_qr_t*          fla_qr_ut_cntl;
fla_qr_t*          fla_qr_ut_cntl2;

fla_qr_t*          fla_qr_ut_cntl_in;
fla_qr_t*          fla_qr_ut_cntl_lapack;
fla_blocksize_t*   fla_qr_ut_var1_bsize;
fla_blocksize_t*   fla_qr_ut_var1_bsize_in;
double             fla_qr_ut_var1_in_to_ou_bsize_ratio = 0.25;

void FLA_QR_UT_cntl_init()
{
	// Set blocksizes with default values for conventional storage.
	fla_qr_ut_var1_bsize    = FLA_Query_blocksizes( FLA_DIMENSION_MIN );
	fla_qr_ut_var1_bsize_in = FLA_Blocksize_create_copy( fla_qr_ut_var1_bsize );
	FLA_Blocksize_scale( fla_qr_ut_var1_bsize_in, fla_qr_ut_var1_in_to_ou_bsize_ratio );

	// Create a control tree to invoke LAPACK.
	fla_qr_ut_cntl_lapack = FLA_Cntl_qr_obj_create( FLA_FLAT,
	                                                FLA_SUBPROBLEM,
	                                                NULL,
	                                                NULL,
	                                                NULL,
	                                                NULL,
	                                                NULL,
	                                                NULL,
	                                                NULL );

	// Create a control tree for small subproblems.
	fla_qr_ut_cntl_in     = FLA_Cntl_qr_obj_create( FLA_FLAT, 
	                                                FLA_BLOCKED_VARIANT1,
	                                                fla_qr_ut_var1_bsize_in,
	                                                fla_qr_ut_cntl_lapack,
	                                                fla_trmm_cntl_blas,
	                                                fla_trmm_cntl_blas,
	                                                fla_trmm_cntl_blas,
	                                                fla_gemm_cntl_blas,
	                                                fla_gemm_cntl_blas );

	// Create a control tree for larger problems with one level of recursion.
	fla_qr_ut_cntl2       = FLA_Cntl_qr_obj_create( FLA_FLAT, 
	                                                FLA_BLOCKED_VARIANT1,
	                                                fla_qr_ut_var1_bsize,
	                                                fla_qr_ut_cntl_in,
	                                                fla_trmm_cntl_blas,
	                                                fla_trmm_cntl_blas,
	                                                fla_trmm_cntl_blas,
	                                                fla_gemm_cntl_blas,
	                                                fla_gemm_cntl_blas );

	// Create a control tree for large problems with no extra recursion.
	fla_qr_ut_cntl        = FLA_Cntl_qr_obj_create( FLA_FLAT, 
	                                                FLA_BLOCKED_VARIANT1,
	                                                fla_qr_ut_var1_bsize,
	                                                fla_qr_ut_cntl_lapack,
	                                                fla_trmm_cntl_blas,
	                                                fla_trmm_cntl_blas,
	                                                fla_trmm_cntl_blas,
	                                                fla_gemm_cntl_blas,
	                                                fla_gemm_cntl_blas );
}

void FLA_QR_UT_cntl_finalize()
{
	FLA_Cntl_obj_free( fla_qr_ut_cntl );
	FLA_Cntl_obj_free( fla_qr_ut_cntl2 );
	FLA_Cntl_obj_free( fla_qr_ut_cntl_in );
	FLA_Cntl_obj_free( fla_qr_ut_cntl_lapack );

	FLA_Blocksize_free( fla_qr_ut_var1_bsize );
	FLA_Blocksize_free( fla_qr_ut_var1_bsize_in );
}

