/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

FLA_Datatype FLASH_Obj_datatype( FLA_Obj H );
int          FLASH_Obj_scalar_length( FLA_Obj H );
int          FLASH_Obj_scalar_width( FLA_Obj H );
int          FLASH_Obj_depth( FLA_Obj H );
int          FLASH_Obj_blocksizes( FLA_Obj H, int* blocksizes );

FLA_Error    FLASH_Obj_create( FLA_Datatype datatype, int m, int n, int depth, int* blocksizes, FLA_Obj* H );
FLA_Error    FLASH_Obj_create_hierarchy( FLA_Datatype datatype, int m, int n, int depth, int* elem_sizes, FLA_Obj flat_matrix, FLA_Obj* H, int id, int depth_overall, int* depth_sizes, int* m_offsets, int* n_offsets );
FLA_Error    FLASH_Obj_create_without_buffer( FLA_Datatype datatype, int m, int n, int depth, int* blocksizes, FLA_Obj* H );

FLA_Error    FLASH_Obj_create_conf_to( FLA_Trans trans, FLA_Obj H_cur, FLA_Obj* H_new );
FLA_Error    FLASH_Obj_create_hier_conf_to_flat( FLA_Trans trans, FLA_Obj F, int depth, int* blocksizes, FLA_Obj* H );
FLA_Error    FLASH_Obj_create_flat_conf_to_hier( FLA_Trans trans, FLA_Obj H, FLA_Obj* F );
FLA_Error    FLASH_Obj_create_hier_copy_of_flat( FLA_Obj F, int depth, int* blocksizes, FLA_Obj* H );
FLA_Error    FLASH_Obj_create_flat_copy_of_hier( FLA_Obj H, FLA_Obj* F );

void         FLASH_Obj_free( FLA_Obj* H );
void         FLASH_Obj_free_hierarchy( FLA_Obj* H );
void         FLASH_Obj_free_without_buffer( FLA_Obj* H );

FLA_Error    FLASH_Obj_attach_buffer( void* buffer, int ldim, FLA_Obj* H );
FLA_Error    FLASH_Obj_attach_buffer_hierarchy( FLA_Obj F, FLA_Obj* H );

FLA_Error    FLASH_Axpy( FLA_Obj alpha, FLA_Obj X, FLA_Obj Y );
FLA_Error    FLASH_Axpy_submatrix_to_global( FLA_Obj alpha, int m, int n, void* buffer, int ldim, int i, int j, FLA_Obj H );
FLA_Error    FLASH_Axpy_global_to_submatrix( FLA_Obj alpha, int i, int j, FLA_Obj H, int m, int n, void* buffer, int ldim );
FLA_Error    FLASH_Axpy_subobject_to_global( FLA_Obj alpha, FLA_Obj F, int i, int j, FLA_Obj H );
FLA_Error    FLASH_Axpy_global_to_subobject( FLA_Obj alpha, int i, int j, FLA_Obj H, FLA_Obj F );

FLA_Error    FLASH_Copy( FLA_Obj H_src, FLA_Obj H_dest );
FLA_Error    FLASH_Copy_submatrix_to_global( int m, int n, void* buffer, int ldim, int i, int j, FLA_Obj H );
FLA_Error    FLASH_Copy_global_to_submatrix( int i, int j, FLA_Obj H, int m, int n, void* buffer, int ldim );
FLA_Error    FLASH_Copy_subobject_to_global( FLA_Obj F, int i, int j, FLA_Obj H );
FLA_Error    FLASH_Copy_global_to_subobject( int i, int j, FLA_Obj H, FLA_Obj F );

FLA_Error    FLASH_Axpy_hierarchy( int direction, FLA_Obj alpha, FLA_Obj F, int i, int j, FLA_Obj* H );
FLA_Error    FLASH_Axpy_hierarchy_r( int direction, FLA_Obj alpha, FLA_Obj F, int i, int j, FLA_Obj* H, FLASH_Obj_queue* queue );
FLA_Error    FLASH_Copy_hierarchy( int direction, FLA_Obj F, int i, int j, FLA_Obj* H );
FLA_Error    FLASH_Copy_hierarchy_r( int direction, FLA_Obj F, int i, int j, FLA_Obj* H, FLASH_Obj_queue* queue );

FLA_Error    FLASH_Obj_flatten( FLA_Obj H, FLA_Obj F );
FLA_Error    FLASH_Obj_hierarchify( FLA_Obj F, FLA_Obj H );

void*        FLASH_Obj_extract_buffer( FLA_Obj H );

FLA_Error    FLASH_Obj_show( char* header, FLA_Obj H, char* elem_format, char* footer );

void         FLASH_print_struct( FLA_Obj H );
void         FLASH_print_struct_helper( FLA_Obj H, int indent );

void         FLASH_Obj_exec( void* func, FLASH_Obj_queue* queue );
void*        FLASH_Obj_exec_parallel( void* arg );
void         FLASH_Obj_push( int direction, FLA_Obj alpha, FLA_Obj F, FLA_Obj H, FLASH_Obj_queue* queue );
