/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

// --- top-level wrapper prototypes --------------------------------------------

FLA_Error FLA_Chol( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_LU_nopiv( FLA_Obj A );
FLA_Error FLA_LU_piv( FLA_Obj A, FLA_Obj p );
FLA_Error FLA_QR( FLA_Obj A, FLA_Obj t, FLA_Obj S );
FLA_Error FLA_QR_UT( FLA_Obj A, FLA_Obj t, FLA_Obj S );
FLA_Error FLA_LQ( FLA_Obj A, FLA_Obj t, FLA_Obj S );
FLA_Error FLA_LQ_UT( FLA_Obj A, FLA_Obj t, FLA_Obj S );
FLA_Error FLA_Hess( FLA_Obj A, FLA_Obj t, int ilo, int ihi );
FLA_Error FLA_Trinv( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A );
FLA_Error FLA_Ttmm( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Sylv( FLA_Trans transa, FLA_Trans transb, FLA_Obj isgn, FLA_Obj A, FLA_Obj B, FLA_Obj C, FLA_Obj scale );
FLA_Error FLA_SPDinv( FLA_Uplo uplo, FLA_Obj A );

FLA_Error FLA_Accum_S( FLA_Direct direct, FLA_Store storev, FLA_Obj A, FLA_Obj t, FLA_Obj S );
FLA_Error FLA_Accum_S_UT( FLA_Direct direct, FLA_Store storev, FLA_Obj A, FLA_Obj t, FLA_Obj S );
FLA_Error FLA_Swap_rows( FLA_Obj A, int k1, int k2, FLA_Obj ipiv );

// --- top-level wrapper Fortran-77 prototypes ---------------------------------

void FLA_F2C( fla_chol_f       )( F_INT *uplo, F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_lu_nopiv_f   )( F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_lu_piv_f     )( F_INT *A, F_INT *p, F_INT *IERROR );
void FLA_F2C( fla_qr_f         )( F_INT *A, F_INT *t, F_INT *S, F_INT *IERROR );
void FLA_F2C( fla_qr_ut_f      )( F_INT *A, F_INT *t, F_INT *S, F_INT *IERROR );
void FLA_F2C( fla_lq_f         )( F_INT *A, F_INT *t, F_INT *S, F_INT *IERROR );
void FLA_F2C( fla_lq_ut_f      )( F_INT *A, F_INT *t, F_INT *S, F_INT *IERROR );
void FLA_F2C( fla_hess_f       )( F_INT *A, F_INT *T, F_INT *ILO, F_INT *IHI, F_INT *IERROR );
void FLA_F2C( fla_trinv_f      )( F_INT *uplo, F_INT *diag, F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_ttmm_f       )( F_INT *uplo, F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_sylv_f       )( F_INT *transa, F_INT *transb, F_INT *isgn, F_INT *A, F_INT *B, F_INT *C, F_INT *scale, F_INT *IERROR );
void FLA_F2C( fla_spdinv_f     )( F_INT *uplo, F_INT *A, F_INT *IERROR );

void FLA_F2C( fla_accum_s_f    )( F_INT *direct, F_INT *storev, F_INT *A, F_INT *t, F_INT *S, F_INT *IERROR );
void FLA_F2C( fla_accum_s_ut_f )( F_INT *direct, F_INT *storev, F_INT *A, F_INT *t, F_INT *S, F_INT *IERROR );
void FLA_F2C( fla_swap_rows_f  )( F_INT *A, F_INT *k1, F_INT *k2, F_INT *ipiv, F_INT *IERROR );

// --- task wrapper prototypes -------------------------------------------------

FLA_Error FLA_Chol_task( FLA_Uplo uplo, FLA_Obj A, fla_chol_t* cntl );
FLA_Error FLA_LU_nopiv_task( FLA_Obj A, fla_lu_t* cntl );
FLA_Error FLA_Trinv_task( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A, fla_trinv_t* cntl );
FLA_Error FLA_Ttmm_task( FLA_Uplo uplo, FLA_Obj A, fla_ttmm_t* cntl );
FLA_Error FLA_Sylv_task( FLA_Trans transa, FLA_Trans transb, FLA_Obj isgn, FLA_Obj A, FLA_Obj B, FLA_Obj C, FLA_Obj scale, fla_sylv_t* cntl );

// --- internal wrapper prototypes ---------------------------------------------

FLA_Error FLA_Chol_internal( FLA_Uplo uplo, FLA_Obj A, fla_chol_t* cntl );
FLA_Error FLA_LU_nopiv_internal( FLA_Obj A, fla_lu_t* cntl );
FLA_Error FLA_LU_piv_internal( FLA_Obj A, FLA_Obj p, fla_lu_t* cntl );
FLA_Error FLA_QR_internal( FLA_Obj A, FLA_Obj t, FLA_Obj S, fla_qr_t* cntl );
FLA_Error FLA_QR_UT_internal( FLA_Obj A, FLA_Obj t, FLA_Obj S, fla_qr_t* cntl );
FLA_Error FLA_LQ_internal( FLA_Obj A, FLA_Obj t, FLA_Obj S, fla_lq_t* cntl );
FLA_Error FLA_LQ_UT_internal( FLA_Obj A, FLA_Obj t, FLA_Obj S, fla_lq_t* cntl );
FLA_Error FLA_Hess_internal( FLA_Obj A, FLA_Obj t, int ilo, int ihi, fla_hess_t* cntl );
FLA_Error FLA_Trinv_internal( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A, fla_trinv_t* cntl );
FLA_Error FLA_Ttmm_internal( FLA_Uplo uplo, FLA_Obj A, fla_ttmm_t* cntl );
FLA_Error FLA_Sylv_internal( FLA_Trans transa, FLA_Trans transb, FLA_Obj isgn, FLA_Obj A, FLA_Obj B, FLA_Obj C, FLA_Obj scale, fla_sylv_t* cntl );
FLA_Error FLA_SPDinv_internal( FLA_Uplo uplo, FLA_Obj A, fla_spdinv_t* cntl );

// --- external wrapper prototypes ---------------------------------------------

FLA_Error FLA_Chol_blk_external( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Chol_unb_external( FLA_Uplo uplo, FLA_Obj A );

FLA_Error FLA_LU_nopiv_blk_external( FLA_Obj A );
FLA_Error FLA_LU_nopiv_unb_external( FLA_Obj A );

FLA_Error FLA_LU_piv_blk_external( FLA_Obj A, FLA_Obj p );
FLA_Error FLA_LU_piv_unb_external( FLA_Obj A, FLA_Obj p );

FLA_Error FLA_QR_blk_external( FLA_Obj A, FLA_Obj t );
FLA_Error FLA_QR_unb_external( FLA_Obj A, FLA_Obj t );

FLA_Error FLA_QR_Accum_S_unb_external( FLA_Obj A, FLA_Obj t, FLA_Obj S );

FLA_Error FLA_QR_UT_blk_external( FLA_Obj A, FLA_Obj t );
FLA_Error FLA_QR_UT_unb_external( FLA_Obj A, FLA_Obj t );

FLA_Error FLA_QR_UT_Accum_S_unb_external( FLA_Obj A, FLA_Obj t, FLA_Obj S );

FLA_Error FLA_LQ_blk_external( FLA_Obj A, FLA_Obj t );
FLA_Error FLA_LQ_unb_external( FLA_Obj A, FLA_Obj t );

FLA_Error FLA_LQ_Accum_S_unb_external( FLA_Obj A, FLA_Obj t, FLA_Obj S );

FLA_Error FLA_LQ_UT_blk_external( FLA_Obj A, FLA_Obj t );
FLA_Error FLA_LQ_UT_unb_external( FLA_Obj A, FLA_Obj t );

FLA_Error FLA_LQ_UT_Accum_S_unb_external( FLA_Obj A, FLA_Obj t, FLA_Obj S );

FLA_Error FLA_Hess_blk_external( FLA_Obj A, FLA_Obj t, int ilo, int ihi );
FLA_Error FLA_Hess_unb_external( FLA_Obj A, FLA_Obj t, int ilo, int ihi );

FLA_Error FLA_Trinv_blk_external( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A );
FLA_Error FLA_Trinv_unb_external( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A );

FLA_Error FLA_Ttmm_blk_external( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Ttmm_unb_external( FLA_Uplo uplo, FLA_Obj A );

FLA_Error FLA_Sylv_blk_external( FLA_Trans transa, FLA_Trans transb, FLA_Obj isgn, FLA_Obj A, FLA_Obj B, FLA_Obj C, FLA_Obj scale );
FLA_Error FLA_Sylv_unb_external( FLA_Trans transa, FLA_Trans transb, FLA_Obj isgn, FLA_Obj A, FLA_Obj B, FLA_Obj C, FLA_Obj scale );

FLA_Error FLA_SPDinv_blk_external( FLA_Uplo uplo, FLA_Obj A );

// --- external wrapper Fortran-77 prototypes ----------------------------------

void FLA_F2C( fla_chol_blk_external_f     )( F_INT *uplo, F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_chol_unb_external_f     )( F_INT *uplo, F_INT *A, F_INT *IERROR );

void FLA_F2C( fla_lu_nopiv_blk_external_f )( F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_lu_nopiv_unb_external_f )( F_INT *A, F_INT *IERROR );

void FLA_F2C( fla_lu_piv_blk_external_f   )( F_INT *A, F_INT *p, F_INT *IERROR );
void FLA_F2C( fla_lu_piv_unb_external_f   )( F_INT *A, F_INT *p, F_INT *IERROR );

void FLA_F2C( fla_qr_blk_external_f       )( F_INT *A, F_INT *t, F_INT *IERROR );
void FLA_F2C( fla_qr_unb_external_f       )( F_INT *A, F_INT *t, F_INT *IERROR );

void FLA_F2C( fla_qr_ut_blk_external_f    )( F_INT *A, F_INT *t, F_INT *IERROR );
void FLA_F2C( fla_qr_ut_unb_external_f    )( F_INT *A, F_INT *t, F_INT *IERROR );

void FLA_F2C( fla_lq_blk_external_f       )( F_INT *A, F_INT *t, F_INT *IERROR );
void FLA_F2C( fla_lq_unb_external_f       )( F_INT *A, F_INT *t, F_INT *IERROR );

void FLA_F2C( fla_lq_ut_blk_external_f    )( F_INT *A, F_INT *t, F_INT *IERROR );
void FLA_F2C( fla_lq_ut_unb_external_f    )( F_INT *A, F_INT *t, F_INT *IERROR );

void FLA_F2C( fla_hrd_blk_external_f      )( F_INT *A, F_INT *T, F_INT *ILO, F_INT *IHI, F_INT *IERROR );
void FLA_F2C( fla_hrd_unb_external_f      )( F_INT *A, F_INT *T, F_INT *ILO, F_INT *IHI, F_INT *IERROR );

void FLA_F2C( fla_trinv_blk_external_f    )( F_INT *uplo, F_INT *diag, F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_trinv_unb_external_f    )( F_INT *uplo, F_INT *diag, F_INT *A, F_INT *IERROR );

void FLA_F2C( fla_ttmm_blk_external_f     )( F_INT *uplo, F_INT *A, F_INT *IERROR );
void FLA_F2C( fla_ttmm_unb_external_f     )( F_INT *uplo, F_INT *A, F_INT *IERROR );

void FLA_F2C( fla_sylv_blk_external_f     )( F_INT *transa, F_INT *transb, F_INT *isgn, F_INT *A, F_INT *B, F_INT *C, F_INT *scale, F_INT *IERROR );
void FLA_F2C( fla_sylv_unb_external_f     )( F_INT *transa, F_INT *transb, F_INT *isgn, F_INT *A, F_INT *B, F_INT *C, F_INT *scale, F_INT *IERROR );

void FLA_F2C( fla_spdinv_blk_external_f   )( F_INT *uplo, F_INT *A, F_INT *IERROR );

// --- check routine prototypes ------------------------------------------------

FLA_Error FLA_Chol_check( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_LU_nopiv_check( FLA_Obj A );
FLA_Error FLA_LU_piv_check( FLA_Obj A, FLA_Obj p );
FLA_Error FLA_QR_check( FLA_Obj A, FLA_Obj t );
FLA_Error FLA_LQ_check( FLA_Obj A, FLA_Obj t );
FLA_Error FLA_Hess_check( FLA_Obj A, FLA_Obj t, int ilo, int ihi );
FLA_Error FLA_Trinv_check( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A );
FLA_Error FLA_Ttmm_check( FLA_Uplo uplo, FLA_Obj A );
FLA_Error FLA_Sylv_check( FLA_Trans transa, FLA_Trans transb, FLA_Obj isgn, FLA_Obj A, FLA_Obj B, FLA_Obj C, FLA_Obj scale );
FLA_Error FLA_SPDinv_check( FLA_Uplo uplo, FLA_Obj A );

FLA_Error FLA_Accum_S_check( FLA_Direct direct, FLA_Store storev, FLA_Obj A, FLA_Obj t, FLA_Obj S );
FLA_Error FLA_Swap_rows_check( FLA_Obj A, int k1, int k2, FLA_Obj ipiv );

FLA_Error FLA_Chol_internal_check( FLA_Uplo uplo, FLA_Obj A, fla_chol_t* cntl );
FLA_Error FLA_LU_nopiv_internal_check( FLA_Obj A, fla_lu_t* cntl );
FLA_Error FLA_Trinv_internal_check( FLA_Uplo uplo, FLA_Diag diag, FLA_Obj A, fla_trinv_t* cntl );
FLA_Error FLA_Ttmm_internal_check( FLA_Uplo uplo, FLA_Obj A, fla_ttmm_t* cntl );
FLA_Error FLA_SPDinv_internal_check( FLA_Uplo uplo, FLA_Obj A, fla_spdinv_t* cntl );
FLA_Error FLA_Sylv_internal_check( FLA_Trans transa, FLA_Trans transb, FLA_Obj isgn, FLA_Obj A, FLA_Obj B, FLA_Obj C, FLA_Obj scale, fla_sylv_t* cntl );

