*
*  libflame
*  An object-based infrastructure for developing high-performance
*  dense linear algebra libraries.
*
*  Copyright (C) 2008, The University of Texas
*
*  libflame is free software; you can redistribute it and/or modify
*  it under the terms of the GNU Lesser General Public License as
*  published by the Free Software Foundation; either version 2.1 of
*  the License, or (at your option) any later version.
*
*  libflame is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with libflame; if you did not receive a copy, see
*  http://www.gnu.org/licenses/.
*
*  For more information, please contact us at flame@cs.utexas.edu or
*  send mail to:
*
*  Field G. Van Zee and/or
*  Robert A. van de Geijn
*  The University of Texas at Austin
*  Department of Computer Sciences
*  1 University Station C0500
*  Austin TX 78712
*
      SUBROUTINE FLA_INIT_FORTRAN( MINUS_ONE_C, ZERO_C, ONE_C, TWO_C )
*
*     Include Fortran FLAME definitions.
*
      include "FLAMEf.h"
*
*     Declare variable-length integer arrays as our incoming scalar
*     constants from C land.
*
      INTEGER MINUS_ONE_C(*), 
     $        ZERO_C(*),
     $        ONE_C(*),
     $        TWO_C(*)
*
*     Declare an integer variable to hold the size of the FLA_Obj in
*     units of float words.
*
      INTEGER NUMWORDS
*
*     Compute the number of words in the FLA_Obj.
*
      NUMWORDS = FLA_OBJ_DATATYPE_SIZE_F( FLA_OBJ ) / 
     $           FLA_OBJ_DATATYPE_SIZE_F( FLA_FLOAT )
*
*     Copy the contents of the C FLA_Obj structures into those declared
*     in FLAMEf.h.
*
      CALL SCOPY( NUMWORDS, MINUS_ONE_C, 1, FLA_MINUS_ONE, 1 )
      CALL SCOPY( NUMWORDS, ZERO_C,      1, FLA_ZERO,      1 )
      CALL SCOPY( NUMWORDS, ONE_C,       1, FLA_ONE,       1 )
      CALL SCOPY( NUMWORDS, TWO_C,       1, FLA_TWO,       1 )

      RETURN
      END


      SUBROUTINE FLA_FINALIZE_FORTRAN()
*
*     Since the scalar constants defined in FLAMEf.h were copied in
*     fla_init_fortran() using a "shallow" copy method, we don't have
*     any storage to free. FLA_Finalize() will take care of the freeing
*     that is necessary.
*
      RETURN
      END
