/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"


/* ***************************************************************************

   FLA_Param_map_to_blas_trans( )

 *************************************************************************** */

void FLA_Param_map_to_blas_trans( FLA_Trans trans, void* blas_trans )
{
	if ( trans == FLA_NO_TRANSPOSE )
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_TRANSPOSE* ) blas_trans ) = CblasNoTrans;
#else
		*( ( char*            ) blas_trans ) = 'N';
#endif
	}
	else if ( trans == FLA_TRANSPOSE )
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_TRANSPOSE* ) blas_trans ) = CblasTrans;
#else
		*( ( char*            ) blas_trans ) = 'T';
#endif
	}
	else if ( trans == FLA_CONJ_TRANSPOSE )
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_TRANSPOSE* ) blas_trans ) = CblasConjTrans;
#else
		*( ( char*            ) blas_trans ) = 'C';
#endif
	}
	else
	{
		FLA_Check_error_code( FLA_NOT_YET_IMPLEMENTED );
	}
}


/* ***************************************************************************

   FLA_Param_map_to_blas_uplo( )

 *************************************************************************** */

void FLA_Param_map_to_blas_uplo( FLA_Uplo uplo, void* blas_uplo )
{
	if ( uplo == FLA_LOWER_TRIANGULAR )
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_UPLO* ) blas_uplo ) = CblasLower;
#else
		*( ( char*       ) blas_uplo ) = 'L';
#endif
	}
	else
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_UPLO* ) blas_uplo ) = CblasUpper;
#else
		*( ( char*       ) blas_uplo ) = 'U';
#endif
	}
}

/* ***************************************************************************

   FLA_Param_map_to_blas_side( )

 *************************************************************************** */

void FLA_Param_map_to_blas_side( FLA_Side side, void* blas_side )
{
	if ( side == FLA_LEFT )
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_SIDE* ) blas_side ) = CblasLeft;
#else
		*( ( char*       ) blas_side ) = 'L';
#endif
	}
	else
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_SIDE* ) blas_side ) = CblasRight;
#else
		*( ( char*       ) blas_side ) = 'R';
#endif
	}
}

/* ***************************************************************************

   FLA_Param_map_to_blas_diag( )

 *************************************************************************** */

void FLA_Param_map_to_blas_diag( FLA_Diag diag, void* blas_diag )
{
	if ( diag == FLA_NONUNIT_DIAG )
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_DIAG* ) blas_diag ) = CblasNonUnit;
#else
		*( ( char*       ) blas_diag ) = 'N';
#endif
	}
	else
	{
#ifdef FLA_ENABLE_CBLAS_INTERFACE
		*( ( CBLAS_DIAG* ) blas_diag ) = CblasUnit;
#else
		*( ( char*       ) blas_diag ) = 'U';
#endif
	}
}

/* ***************************************************************************

   FLA_Param_map_to_lapack_direct( )

 *************************************************************************** */

void FLA_Param_map_to_lapack_direct( FLA_Direct direct, void* lapack_direct )
{
	if ( direct == FLA_FORWARD )
	{
		*( ( char* ) lapack_direct ) = 'F';
	}
	else // direct == FLA_BACKWARD
	{
		*( ( char* ) lapack_direct ) = 'B';
	}
}

/* ***************************************************************************

   FLA_Param_map_to_lapack_storev( )

 *************************************************************************** */

void FLA_Param_map_to_lapack_storev( FLA_Store storev, void* lapack_storev )
{
	if ( storev == FLA_COLUMNWISE )
	{
		*( ( char* ) lapack_storev ) = 'C';
	}
	else // storev == FLA_ROWWISE
	{
		*( ( char* ) lapack_storev ) = 'R';
	}
}

