/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Random_spd_matrix( FLA_Uplo uplo, FLA_Obj A )
{
  FLA_Obj R;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Random_spd_matrix_check( uplo, A );
#endif

  // Create a temporary object R conformal to A.
  FLA_Obj_create_conf_to( FLA_NO_TRANSPOSE, A, &R );

  if ( FLA_Obj_is_real( A ) )
  {
    // Fill R with random real entries.
    FLA_Random_matrix( R );
    
    // A = R * R';
    FLA_Syrk_external( uplo, FLA_NO_TRANSPOSE, FLA_ONE, R, FLA_ZERO, A );
  }
  else
  {
    // Fill R with random complex entries.
    FLA_Random_matrix( R );
    
    // A = R * conjg( R' );
    FLA_Herk_external( uplo, FLA_NO_TRANSPOSE, FLA_ONE, R, FLA_ZERO, A );
  }

  // Free R.
  FLA_Obj_free( &R );

  // Make A symmetric, preserving the uplo triangle.
  FLA_Symmetrize( uplo, A );

  return FLA_SUCCESS;
}

void FLA_F2C( fla_random_spd_matrix_f ) 
     ( F_INT *uplo, F_INT *A, F_INT *IERROR )
{
  *IERROR = FLA_Random_spd_matrix( *( ( FLA_Uplo * ) uplo ),
                                   *( ( FLA_Obj  * ) A    ) );
}

