/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Shift_pivots_to( FLA_Pivot_type ptype, FLA_Obj ipiv )
{
  int          m_ipiv, n_ipiv;
  int*         buff_ipiv;
  int          i;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Shift_pivots_to_check( ptype, ipiv );
#endif

  m_ipiv    = FLA_Obj_length( ipiv );
  n_ipiv    = FLA_Obj_width( ipiv );
  buff_ipiv = FLA_INT_PTR( ipiv );

  if ( m_ipiv < 1 || n_ipiv < 1 ) return FLA_SUCCESS;

  if ( ptype == FLA_LAPACK_PIVOTS )
  {
    // Shift FLAME pivots to LAPACK pivots.
    for ( i = 0; i < m_ipiv; i++ )
      buff_ipiv[ i ] += i + 1;
  }
  else
  {
    // Otherwise, shift LAPACK pivots back to FLAME.
    for ( i = 0; i < m_ipiv; i++ )
      buff_ipiv[ i ] -= i + 1;
  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_shift_pivots_to_f ) 
     ( F_INT *ptype, F_INT *ipiv, F_INT *IERROR )
{
  *IERROR = FLA_Shift_pivots_to( *( ( FLA_Pivot_type * ) ptype ),
                                 *( ( FLA_Obj        * ) ipiv  ) );
}

