/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Store_S( FLA_Obj T, FLA_Obj S )
{
  FLA_Obj     
    TTL, TTR,   T00, T01, T02,
    TBL, TBR,   T10, T11, T12,
                T20, T21, T22;
  FLA_Obj
    ST,   S0,
    SB,   S1,   S1L, S1R,
          S2;
  int
    b, nb_alg;

  nb_alg = FLA_Obj_width( S );

  FLA_Part_2x2( T,  &TTL, &TTR,
                    &TBL, &TBR,     0, 0,  FLA_TL );

  FLA_Part_2x1( S,  &ST,
                    &SB,            0, FLA_TOP );

  while ( FLA_Obj_min_dim( TBR ) > 0 ){

    b = min( FLA_Obj_min_dim( TBR ), nb_alg );

    FLA_Repart_2x2_to_3x3( TTL, /**/ TTR,      &T00, /**/ &T01, &T02,
                        /* ************* */ /* ********************* */
                                /**/           &T10, /**/ &T11, &T12,
                           TBL, /**/ TBR,      &T20, /**/ &T21, &T22, 
                           b, b, FLA_BR );

    FLA_Repart_2x1_to_3x1( ST,               &S0,
                        /* ** */           /* ** */
                                             &S1,
                           SB,               &S2,      b, FLA_BOTTOM );

    /*------------------------------------------------------------*/

    if ( b != nb_alg )
    {
      FLA_Part_1x2( S1, &S1L, &S1R, b, FLA_LEFT );
   
      FLA_Copy( T11, S1L );
    }
    else
      FLA_Copy( T11, S1 );

    /*------------------------------------------------------------*/

    FLA_Cont_with_3x3_to_2x2( &TTL, /**/ &TTR,      T00, T01, /**/ T02,
                                    /**/            T10, T11, /**/ T12,
                            /* ************** */ /* ****************** */
                              &TBL, /**/ &TBR,      T20, T21, /**/ T22, 
                              FLA_TL );

    FLA_Cont_with_3x1_to_2x1( &ST,                S0,
                                                  S1,
                            /* ** */           /* ** */
                              &SB,                S2,     FLA_TOP );
  }
  
  return FLA_SUCCESS;
}

