/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Axpy_s( FLA_Obj alpha0, FLA_Obj alpha1, FLA_Obj A, FLA_Obj beta, FLA_Obj B )
{
  FLA_Obj   A_copy;
  FLA_Obj   alpha_copy;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() )
    FLA_Axpy_s_check( alpha0, alpha1, A, beta, B );
#endif

  if ( FLA_Obj_equals( beta, FLA_ONE ) == FALSE  )
    FLA_Scal( beta, B );

  if ( FLA_Obj_equals( alpha0, FLA_ZERO ) || 
       FLA_Obj_equals( alpha1, FLA_ZERO ) )
    return FLA_SUCCESS;

  if ( FLA_Obj_is_vector( A ) )
  {
    FLA_Obj_create( FLA_Obj_datatype( A ), 1, 1, &alpha_copy );

    FLA_Copy( alpha1, alpha_copy );

    FLA_Scal( alpha0, alpha_copy );

    FLA_Axpy( alpha_copy, A, B );

    FLA_Obj_free( &alpha_copy );
  }
  else
  {
    FLA_Obj_create_conf_to( FLA_NO_TRANSPOSE, A, &A_copy );

    FLA_Copy_t( FLA_NO_TRANSPOSE, A, A_copy );

    FLA_Scal( alpha0, A_copy );

    FLA_Axpy( alpha1, A_copy, B );

    FLA_Obj_free( &A_copy );
  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_axpy_s_f )
     ( F_INT *alpha0, F_INT *alpha1, F_INT *A, F_INT *beta, F_INT *B, F_INT *IERROR )
{
  *IERROR = FLA_Axpy_s( *( ( FLA_Obj * ) alpha0 ),
                        *( ( FLA_Obj * ) alpha1 ),
                        *( ( FLA_Obj * ) A      ),
                        *( ( FLA_Obj * ) beta   ),
                        *( ( FLA_Obj * ) B      ) );
}

