/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Axpy_t( FLA_Trans trans, FLA_Obj alpha, FLA_Obj A, FLA_Obj B )
{
  FLA_Obj   A_copy;
  FLA_Trans trans_real;
  FLA_Trans trans_conj;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() ) 
    FLA_Axpy_t_check( trans, alpha, A, B );
#endif

  if ( FLA_Obj_is_vector( A ) )
  {
    trans_conj = FLA_TRANS_MAP_TO_NC( trans );

    FLA_Obj_create_conf_to( FLA_NO_TRANSPOSE, A, &A_copy );

    FLA_Copy_t( trans_conj, A, A_copy );

    FLA_Axpy( alpha, A_copy, B );

    FLA_Obj_free( &A_copy );
  }
  else
  {
    trans_real = FLA_TRANS_MAP_TO_NT( trans );

    FLA_Obj_create_conf_to( trans_real, A, &A_copy );

    FLA_Copy_t( trans, A, A_copy );

    FLA_Axpy( alpha, A_copy, B );

    FLA_Obj_free( &A_copy );
  }

  return FLA_SUCCESS;
}

void FLA_F2C( fla_axpy_t_f ) 
     ( F_INT *trans, F_INT *alpha, F_INT *A, F_INT *B, F_INT *IERROR )
{
  *IERROR = FLA_Axpy_t( *( ( FLA_Trans * ) trans ), 
                        *( ( FLA_Obj   * ) alpha ), 
                        *( ( FLA_Obj   * ) A     ), 
                        *( ( FLA_Obj   * ) B     ) );
}

