/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Trsv_sx( FLA_Uplo uplo, FLA_Trans transa, FLA_Diag diag, FLA_Obj alpha, FLA_Obj A, FLA_Obj x, FLA_Obj beta, FLA_Obj y ) 
{
  FLA_Obj x_copy;

#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  if ( FLA_Check_parameters() ) 
    FLA_Trsv_sx_check( uplo, transa, diag, alpha, A, x, beta, y );
#endif

  FLA_Obj_create_conf_to( FLA_NO_TRANSPOSE, x, &x_copy );
  
  FLA_Copy( x, x_copy );

  FLA_Trsv( uplo, transa, diag, A, x_copy );

  FLA_Scal( beta, y );
  
  FLA_Axpy( alpha, x_copy, y );

  FLA_Obj_free( &x_copy );

  return FLA_SUCCESS;
}

void FLA_F2C( fla_trsv_sx_f ) 
     ( F_INT *uplo, F_INT *transa, F_INT *diag, F_INT *alpha, F_INT *A, F_INT *x, F_INT *beta, F_INT *y, F_INT *IERROR )
{
  *IERROR = FLA_Trsv_sx( *( ( FLA_Uplo  * ) uplo   ), 
                        *( ( FLA_Trans * ) transa ),
                        *( ( FLA_Diag  * ) diag   ),
                        *( ( FLA_Obj   * ) alpha  ),
                        *( ( FLA_Obj   * ) A      ),
                        *( ( FLA_Obj   * ) x      ),
                        *( ( FLA_Obj   * ) beta   ),
                        *( ( FLA_Obj   * ) y      ) );
}

