/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


PLA_Error PLA_Local_gemv_check( PLA_Trans transa,
				PLA_Obj   alpha,
				PLA_Obj   A,
				PLA_Obj   x,
				PLA_Obj   beta,
				PLA_Obj   y )
/*----------------------------------------------------------------------------

   PLA_Local_gemv_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, y, "PLA_Local_gemv: objects not on same processor" );
      PLA_Check_consistent_owners( A,     y, "PLA_Local_gemv: objects not on same processor" );
      PLA_Check_consistent_owners( x,     y, "PLA_Local_gemv: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  y, "PLA_Local_gemv: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_ger_check( PLA_Obj alpha,
			       PLA_Obj x,
			       PLA_Obj y,
			       PLA_Obj A )
/*----------------------------------------------------------------------------

   PLA_Local_ger_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, A, "PLA_Local_ger: objects not on same processor" );
      PLA_Check_consistent_owners( x,     A, "PLA_Local_ger: objects not on same processor" );
      PLA_Check_consistent_owners( y,     A, "PLA_Local_ger: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_gerc_check( PLA_Obj alpha,
				PLA_Obj x,
				PLA_Obj y,
				PLA_Obj A )
/*----------------------------------------------------------------------------

   PLA_Local_gerc_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, A, "PLA_Local_gerc: objects not on same processor" );
      PLA_Check_consistent_owners( x,     A, "PLA_Local_gerc: objects not on same processor" );
      PLA_Check_consistent_owners( y,     A, "PLA_Local_gerc: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_hemv_check( PLA_Uplo uplo,
				PLA_Obj  alpha,
				PLA_Obj  A,
				PLA_Obj  x,
				PLA_Obj  beta,
				PLA_Obj  y )
/*----------------------------------------------------------------------------

   PLA_Local_hemv_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, y, "PLA_Local_hemv: objects not on same processor" );
      PLA_Check_consistent_owners( A,     y, "PLA_Local_hemv: objects not on same processor" );
      PLA_Check_consistent_owners( x,     y, "PLA_Local_hemv: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  y, "PLA_Local_hemv: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_her_check( PLA_Uplo uplo,
			       PLA_Obj  alpha,
			       PLA_Obj  x,
			       PLA_Obj  A )
/*----------------------------------------------------------------------------

   PLA_Local_her_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, A, "PLA_Local_her: objects not on same processor" );
      PLA_Check_consistent_owners( x,     A, "PLA_Local_her: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_her2_check( PLA_Uplo uplo,
				PLA_Obj  alpha,
				PLA_Obj  x,
				PLA_Obj  y,
				PLA_Obj  A )
/*----------------------------------------------------------------------------

   PLA_Local_her2_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, A, "PLA_Local_her2: objects not on same processor" );
      PLA_Check_consistent_owners( x,     A, "PLA_Local_her2: objects not on same processor" );
      PLA_Check_consistent_owners( y,     A, "PLA_Local_her2: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_symv_check( PLA_Uplo uplo,
				PLA_Obj  alpha,
				PLA_Obj  A,
				PLA_Obj  x,
				PLA_Obj  beta,
				PLA_Obj  y )
/*----------------------------------------------------------------------------

   PLA_Local_symv_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, y, "PLA_Local_symv: objects not on same processor" );
      PLA_Check_consistent_owners( A,     y, "PLA_Local_symv: objects not on same processor" );
      PLA_Check_consistent_owners( x,     y, "PLA_Local_symv: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  y, "PLA_Local_symv: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_syr_check( PLA_Uplo uplo,
			       PLA_Obj  alpha,
			       PLA_Obj  x,
			       PLA_Obj  A )
/*----------------------------------------------------------------------------

   PLA_Local_syr_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, A, "PLA_Local_syr: objects not on same processor" );
      PLA_Check_consistent_owners( x,     A, "PLA_Local_syr: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_syr2_check( PLA_Uplo uplo,
				PLA_Obj  alpha,
				PLA_Obj  x,
				PLA_Obj  y,
				PLA_Obj  A )
/*----------------------------------------------------------------------------

   PLA_Local_syr2_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, A, "PLA_Local_syr2: objects not on same processor" );
      PLA_Check_consistent_owners( x,     A, "PLA_Local_syr2: objects not on same processor" );
      PLA_Check_consistent_owners( y,     A, "PLA_Local_syr2: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_trmv_check( PLA_Uplo  uplo,
				PLA_Trans transa,
				PLA_Diag  diag,
				PLA_Obj   A,
				PLA_Obj   x )
/*----------------------------------------------------------------------------

   PLA_Local_trmv_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( A, x, "PLA_Local_trmv: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_trmv_sx_check( PLA_Uplo  uplo,
				  PLA_Trans transa,
				  PLA_Diag  diag,
				  PLA_Obj   alpha,
				  PLA_Obj   A,
				  PLA_Obj   x,
				  PLA_Obj   beta,
				  PLA_Obj   y )
/*----------------------------------------------------------------------------

   PLA_Local_trmv_sx_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, y, "PLA_Local_trmv_sx: objects not on same processor" );
      PLA_Check_consistent_owners( A,     y, "PLA_Local_trmv_sx: objects not on same processor" );
      PLA_Check_consistent_owners( x,     y, "PLA_Local_trmv_sx: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  y, "PLA_Local_trmv_sx: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_trsv_check( PLA_Uplo  uplo,
				PLA_Trans transa,
				PLA_Diag  diag,
				PLA_Obj   A,
				PLA_Obj   x )
/*----------------------------------------------------------------------------

   PLA_Local_trsv_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( A, x, "PLA_Local_trsv: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_trsv_sx_check( PLA_Uplo  uplo,
				  PLA_Trans transa,
				  PLA_Diag  diag,
				  PLA_Obj   alpha,
				  PLA_Obj   A,
				  PLA_Obj   x,
				  PLA_Obj   beta,
				  PLA_Obj   y )
/*----------------------------------------------------------------------------

   PLA_Local_trsv_sx_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, y, "PLA_Local_trsv_sx: objects not on same processor" );
      PLA_Check_consistent_owners( A,     y, "PLA_Local_trsv_sx: objects not on same processor" );
      PLA_Check_consistent_owners( x,     y, "PLA_Local_trsv_sx: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  y, "PLA_Local_trsv_sx: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}
