/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


PLA_Error PLA_Local_gemm_check( PLA_Trans transa,
				PLA_Trans transb,
				PLA_Obj   alpha,
				PLA_Obj   A,
				PLA_Obj   B,
				PLA_Obj   beta,
				PLA_Obj   C )
/*----------------------------------------------------------------------------

   PLA_Local_gemm_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_gemm: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_gemm: objects not on same processor" );
      PLA_Check_consistent_owners( B,     C, "PLA_Local_gemm: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_gemm: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_hemm_check( PLA_Side side,
				PLA_Uplo uplo,
				PLA_Obj  alpha,
				PLA_Obj  A,
				PLA_Obj  B,
				PLA_Obj  beta,
				PLA_Obj  C )
/*----------------------------------------------------------------------------

   PLA_Local_hemm_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_hemm: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_hemm: objects not on same processor" );
      PLA_Check_consistent_owners( B,     C, "PLA_Local_hemm: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_hemm: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_her2k_check( PLA_Uplo  uplo,
				 PLA_Trans trans,
				 PLA_Obj   alpha,
				 PLA_Obj   A,
				 PLA_Obj   B,
				 PLA_Obj   beta,
				 PLA_Obj   C )
/*----------------------------------------------------------------------------

   PLA_Local_her2k_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_her2k: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_her2k: objects not on same processor" );
      PLA_Check_consistent_owners( B,     C, "PLA_Local_her2k: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_her2k: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_herk_check( PLA_Uplo  uplo,
				PLA_Trans trans,
				PLA_Obj   alpha,
				PLA_Obj   A,
				PLA_Obj   beta,
				PLA_Obj   C )
/*----------------------------------------------------------------------------

   PLA_Local_herk_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_herk: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_herk: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_herk: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_symm_check( PLA_Side side,
				PLA_Uplo uplo,
				PLA_Obj  alpha,
				PLA_Obj  A,
				PLA_Obj  B,
				PLA_Obj  beta,
				PLA_Obj  C )
/*----------------------------------------------------------------------------

   PLA_Local_symm_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_symm: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_symm: objects not on same processor" );
      PLA_Check_consistent_owners( B,     C, "PLA_Local_symm: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_symm: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_syr2k_check( PLA_Uplo  uplo,
				 PLA_Trans trans,
				 PLA_Obj   alpha,
				 PLA_Obj   A,
				 PLA_Obj   B,
				 PLA_Obj   beta,
				 PLA_Obj   C )
/*----------------------------------------------------------------------------

   PLA_Local_syr2k_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_syr2k: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_syr2k: objects not on same processor" );
      PLA_Check_consistent_owners( B,     C, "PLA_Local_syr2k: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_syr2k: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_syrk_check( PLA_Uplo  uplo,
			        PLA_Trans trans,
				PLA_Obj   alpha,
				PLA_Obj   A,
				PLA_Obj   beta,
				PLA_Obj   C )
/*----------------------------------------------------------------------------

   PLA_Local_syrk_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_syrk: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_syrk: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_syrk: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_trmm_check( PLA_Side  side,
				PLA_Uplo  uplo,
			        PLA_Trans trans,
				PLA_Diag  diag,
				PLA_Obj   alpha,
				PLA_Obj   A,
				PLA_Obj   B )
/*----------------------------------------------------------------------------

   PLA_Local_trmm_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, B, "PLA_Local_trmm: objects not on same processor" );
      PLA_Check_consistent_owners( A,     B, "PLA_Local_trmm: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_trmm_sx_check( PLA_Side  side,
				  PLA_Uplo  uplo,
				  PLA_Trans trans,
				  PLA_Diag  diag,
				  PLA_Obj   alpha,
				  PLA_Obj   A,
				  PLA_Obj   B,
				  PLA_Obj   beta,
				  PLA_Obj   C )
/*----------------------------------------------------------------------------

   PLA_Local_trmm_sx_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_trmm_sx: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_trmm_sx: objects not on same processor" );
      PLA_Check_consistent_owners( B,     C, "PLA_Local_trmm_sx: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_trmm_sx: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_trsm_check( PLA_Side  side,
				PLA_Uplo  uplo,
				PLA_Trans trans,
				PLA_Diag  diag,
				PLA_Obj   alpha,
				PLA_Obj   A,
				PLA_Obj   B )
/*----------------------------------------------------------------------------

   PLA_Local_trsm_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, B, "PLA_Local_trsm: objects not on same processor" );
      PLA_Check_consistent_owners( A,     B, "PLA_Local_trsm: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}


PLA_Error PLA_Local_trsm_sx_check( PLA_Side  side,
				  PLA_Uplo  uplo,
				  PLA_Trans trans,
				  PLA_Diag  diag,
				  PLA_Obj   alpha,
				  PLA_Obj   A,
				  PLA_Obj   B,
				  PLA_Obj   beta,
				  PLA_Obj   C )
/*----------------------------------------------------------------------------

   PLA_Local_trsm_sx_check

----------------------------------------------------------------------------*/
{
   if ( PLA_Check_parameters( ) )
   {
      PLA_Check_consistent_owners( alpha, C, "PLA_Local_trsm_sx: objects not on same processor" );
      PLA_Check_consistent_owners( A,     C, "PLA_Local_trsm_sx: objects not on same processor" );
      PLA_Check_consistent_owners( B,     C, "PLA_Local_trsm_sx: objects not on same processor" );
      PLA_Check_consistent_owners( beta,  C, "PLA_Local_trsm_sx: objects not on same processor" );
   }
   
   return PLA_SUCCESS;
}
