/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


PLA_Error PLA_Local_cholesky( PLA_Obj );


PLA_Error PLA_Local_cholesky( PLA_Obj A )
{
   PLA_Error info = PLA_SUCCESS;
   
   PLA_Obj 
      ATL,   ATR,      A00,  a01,     A02, 
      ABL,   ABR,      a10t, alpha11, a12t,
                       A20,  a21,     A22;
  
   PLA_Part_2x2( A,    &ATL, &ATR,
		       &ABL, &ABR,     0, 0, PLA_TL );

   while ( PLA_Obj_length( ATL ) < PLA_Obj_length( A ) )
   {
      PLA_Repart_2x2_to_3x3( ATL, /**/ ATR,      &A00,  /**/ &a01,     &A02,
                          /* ************* */ /* ************************** */
                                                 &a10t, /**/ &alpha11, &a12t,
			     ABL, /**/ ABR,      &A20,  /**/ &a21,     &A22,
			     1, 1, PLA_BR );
      
      /*------------------------------------------------------------*/
      
      PLA_Local_dot_x( PLA_MINUS_ONE, a10t, a10t, PLA_ONE, alpha11 );
      
      info += PLA_Sqrt( alpha11 );
      
      PLA_Local_gemv( PLA_NO_TRANSPOSE, PLA_MINUS_ONE, 
		      A20, a10t, PLA_ONE, a21 );
      
      PLA_Local_inv_scal( alpha11, a21 );
      
      /*------------------------------------------------------------*/
      
      PLA_Cont_with_3x3_to_2x2( &ATL, /**/ &ATR,      A00,  a01,     /**/ A02,
                                                      a10t, alpha11, /**/ a12t,
                             /* *************** */ /* ********************** */
				&ABL, /**/ &ABR,      A20,  a21,     /**/ A22,
				PLA_TL );
   }
   
   return info;
}
