/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


PLA_Error alg( PLA_Obj );


PLA_Error alg( PLA_Obj A )
{
   int b;
   
   PLA_Obj 
      ATL, ATR,   A00, A01, A02,
      ABL, ABR,   A10, A11, A12,
                  A20, A21, A22;
  
   PLA_Part_2x2( A,   &ATL, &ATR,
		      &ABL, &ABR,
		 0, 0, PLA_TL );
   
   while ( PLA_Obj_length( ATL ) < PLA_Obj_length( A ) )
   {
      b = PLA_Obj_split_size( ABR, PLA_TL );
      
      PLA_Repart_2x2_to_3x3( ATL, /**/ ATR,       &A00, /**/ &A01, &A02,
			  /* ************** */ /* ********************** */
                                                  &A10, /**/ &A11, &A12,
			     ABL, /**/ ABR,       &A20, /**/ &A21, &A22,
			     b, b, PLA_BR );
      
      /*------------------------------------------------------------*/

      PLA_printf( "b = %d\n", b );

      /*------------------------------------------------------------*/

      PLA_Cont_with_3x3_to_2x2( &ATL, /**/ &ATR,       A00, A01, /**/ A02,
                                                       A10, A11, /**/ A12,
			     /* **************** */ /* ******************* */
				&ABL, /**/ &ABR,       A20, A21, /**/ A22,
				PLA_TL );
   }
   
   return PLA_SUCCESS;
}
