/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "PLA.h"


PLA_Error PLA_Local_chol( PLA_Uplo uplo, 
			  PLA_Obj  A )
/*----------------------------------------------------------------------------

   PLA_Local_asum

----------------------------------------------------------------------------*/
{
   if ( PLA_Error_check( ) )
      PLA_Local_chol_check( uplo, A );

   if ( PLA_Obj_is_owner( A ) )
      FLA_Chol( uplo, A );

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_LU_nopiv( PLA_Obj A )
/*----------------------------------------------------------------------------

   PLA_Local_asum

----------------------------------------------------------------------------*/
{
   if ( PLA_Error_check( ) )
      PLA_Local_LU_nopiv_check( A );
	 
   if ( PLA_Obj_is_owner( A ) )
      FLA_LU_nopiv( A );

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_LU_piv( PLA_Obj A, 
			    PLA_Obj p )
/*----------------------------------------------------------------------------

   PLA_Local_asum

----------------------------------------------------------------------------*/
{
   if ( PLA_Error_check( ) )
      PLA_Local_LU_piv_check( A, p );

   if ( PLA_Obj_is_owner( A ) &&
	PLA_Obj_is_owner( p ) )
      FLA_LU_piv( A, p );

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_QR( PLA_Obj A, 
			PLA_Obj t, 
			PLA_Obj S )
/*----------------------------------------------------------------------------

   PLA_Local_asum

----------------------------------------------------------------------------*/
{
   if ( PLA_Error_check( ) )
      PLA_Local_QR_check( A, t, S );
   
   if ( PLA_Obj_is_owner( A ) &&
	PLA_Obj_is_owner( t ) &&
	PLA_Obj_is_owner( S ) )
      FLA_QR( A, t, S );

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_LQ( PLA_Obj A, 
			PLA_Obj t,
			PLA_Obj S )
/*----------------------------------------------------------------------------

   PLA_Local_asum

----------------------------------------------------------------------------*/
{
   if ( PLA_Error_check( ) )
      PLA_Local_LQ_check( A, t, S );

   if ( PLA_Obj_is_owner( A ) &&
	PLA_Obj_is_owner( t ) &&
	PLA_Obj_is_owner( S ) )
      FLA_LQ( A, t, S );

   return PLA_SUCCESS;
}


PLA_Error PLA_Local_HRD( PLA_Obj A, 
			 PLA_Obj t, 
			 int     ilo, 
			 int     ihi )
/*----------------------------------------------------------------------------

   PLA_Local_asum

----------------------------------------------------------------------------*/
{
   if ( PLA_Error_check( ) )
      PLA_Local_HRD_check( A, t, ilo, ihi );
	 
   if ( PLA_Obj_is_owner( A ) &&
	PLA_Obj_is_owner( t ) )
      FLA_HRD( A, t, ilo, ihi );

   return PLA_SUCCESS;
}
