/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

extern fla_gemm_t* flash_gemm_cntl_mm_op;

FLA_Error FLASH_Gemm( FLA_Trans transa, FLA_Trans transb, FLA_Obj alpha, FLA_Obj A, FLA_Obj B, FLA_Obj beta, FLA_Obj C )
{
  FLA_Error r_val;
#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  FLA_Bool  check_params;
  
  // Check parameters.
  if ( FLA_Check_parameters() )
    FLA_Gemm_check( transa, transb, alpha, A, B, beta, C );

  // Temporarily disable parameter checking.
  check_params = FLA_Check_parameters_set( FALSE );
#endif

  // Begin a parallel region.
  FLASH_Queue_begin();
  
  // Enqueue tasks via a SuperMatrix-aware control tree.
  r_val = FLA_Gemm_internal( transa, transb, alpha, A, B, beta, C, flash_gemm_cntl_mm_op );
  
  // End the parallel region.
  FLASH_Queue_end();
  
#ifdef FLA_ENABLE_INTERNAL_ERROR_CHECKING
  // Restore parameter checking to its previous status.
  FLA_Check_parameters_set( check_params );
#endif

  return r_val;
}

void FLA_F2C( flash_gemm_f ) 
     ( F_INT *transa, F_INT *transb, F_INT *alpha, F_INT *A, F_INT *B, F_INT *beta, F_INT *C, F_INT *IERROR )
{
  *IERROR = FLASH_Gemm( *( ( FLA_Trans * ) transa ),
                        *( ( FLA_Trans * ) transb ),
                        *( ( FLA_Obj   * ) alpha  ),
                        *( ( FLA_Obj   * ) A      ),
                        *( ( FLA_Obj   * ) B      ),
                        *( ( FLA_Obj   * ) beta   ),
                        *( ( FLA_Obj   * ) C      ) );
}

