/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

FLA_Error FLA_Accum_S_blk_var1( FLA_Obj U, FLA_Obj s, FLA_Obj S, int nb_alg )
{
  FLA_Obj UTL,   UTR,      U00, U01, U02, 
          UBL,   UBR,      U10, U11, U12,
                           U20, U21, U22;

  FLA_Obj sT,              s0,
          sB,              s1,
                           s2;

  FLA_Obj STL,   STR,      S00, S01, S02, 
          SBL,   SBR,      S10, S11, S12,
                           S20, S21, S22;

  FLA_Obj UBR1, UBR2;

  int b;

  FLA_Part_2x2( U,    &UTL, &UTR,
                      &UBL, &UBR,     0, 0, FLA_TL );

  FLA_Part_2x1( s,    &sT, 
                      &sB,            0, FLA_TOP );

  FLA_Part_2x2( S,    &STL, &STR,
                      &SBL, &SBR,     0, 0, FLA_TL );

  while ( FLA_Obj_width( UTL ) != FLA_Obj_width( U ) ){

    b = min( FLA_Obj_width( UBR ), nb_alg );

    FLA_Repart_2x2_to_3x3( UTL, /**/ UTR,       &U00, /**/ &U01, &U02,
                        /* ************* */   /* ******************** */
                                                &U10, /**/ &U11, &U12,
                           UBL, /**/ UBR,       &U20, /**/ &U21, &U22,
                           b, b, FLA_BR );

    FLA_Repart_2x1_to_3x1( sT,                &s0, 
                        /* ** */            /* ** */
                                              &s1, 
                           sB,                &s2,        b, FLA_BOTTOM );

    FLA_Repart_2x2_to_3x3( STL, /**/ STR,       &S00, /**/ &S01, &S02,
                        /* ************* */   /* ******************** */
                                                &S10, /**/ &S11, &S12,
                           SBL, /**/ SBR,       &S20, /**/ &S21, &S22,
                           b, b, FLA_BR );

    /*------------------------------------------------------------*/

    FLA_Part_1x2( UBR,   &UBR1, &UBR2, b, FLA_LEFT );

    /* Update S
    Compute S11 from UBR1 and s1
    S01  = - triu( S00 ) * ( U10' * trilu( U11 )+ U20' * U21 ) * S11;
    */

    FLA_Accum_S_unb_var1( UBR1, s1, S11 );
    FLA_Copy_t( FLA_TRANSPOSE, U10, S01 );
    FLA_Trmm_external( FLA_RIGHT, FLA_LOWER_TRIANGULAR, 
                       FLA_NO_TRANSPOSE, FLA_UNIT_DIAG,
                       FLA_ONE, U11, S01 );
    FLA_Gemm_external( FLA_TRANSPOSE, FLA_NO_TRANSPOSE, 
                       FLA_ONE, U20, U21, FLA_ONE, S01 );
    FLA_Trmm_external( FLA_LEFT, FLA_UPPER_TRIANGULAR, 
                       FLA_NO_TRANSPOSE, FLA_NONUNIT_DIAG,
                       FLA_MINUS_ONE, S00, S01 );
    FLA_Trmm_external( FLA_RIGHT, FLA_UPPER_TRIANGULAR, 
                       FLA_NO_TRANSPOSE, FLA_NONUNIT_DIAG,
                       FLA_ONE, S11, S01 );

    /*------------------------------------------------------------*/

    FLA_Cont_with_3x3_to_2x2( &UTL, /**/ &UTR,       U00, U01, /**/ U02,
                                                     U10, U11, /**/ U12,
                            /* ************** */  /* ****************** */
                              &UBL, /**/ &UBR,       U20, U21, /**/ U22,
                              FLA_TL );

    FLA_Cont_with_3x1_to_2x1( &sT,                s0, 
                                                  s1, 
                            /* ** */           /* ** */
                              &sB,                s2,     FLA_TOP );

    FLA_Cont_with_3x3_to_2x2( &STL, /**/ &STR,       S00, S01, /**/ S02,
                                                     S10, S11, /**/ S12,
                            /* ************** */  /* ****************** */
                              &SBL, /**/ &SBR,       S20, S21, /**/ S22,
                              FLA_TL );

  }

  return FLA_SUCCESS;
}
