/*
   libflame
   An object-based infrastructure for developing high-performance
   dense linear algebra libraries.

   Copyright (C) 2008, The University of Texas

   libflame is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   libflame is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with libflame; if you did not receive a copy, see
   http://www.gnu.org/licenses/.

   For more information, please contact us at flame@cs.utexas.edu or
   send mail to:

   Field G. Van Zee and/or
   Robert A. van de Geijn
   The University of Texas at Austin
   Department of Computer Sciences
   1 University Station C0500
   Austin TX 78712
*/

#include "FLAME.h"

#define A(i,j) buf_A[ (j)*ldim_A + (i) ]
#define S(i,j) buf_S[ (j)*ldim_S + (i) ]
#define t(i) buf_t[ i ]

FLA_Error FLA_QR_Accum_S_unb_var2_ugly( FLA_Obj A, FLA_Obj t, FLA_Obj S )
{
  int m, n, j, ldim_A, ldim_S, m_min_j_min_1, m_min_j, i_one = 1;
  double *buf_A, *buf_t, *buf_S, *buf_w01, d_one = 1.0, d_minus_one = -1.0;

  m = FLA_Obj_length( A ); 
  n = FLA_Obj_width( A ); 
  ldim_A = FLA_Obj_ldim( A );
  ldim_S = FLA_Obj_ldim( S );
  buf_A = ( double * ) FLA_Obj_buffer( A );
  buf_S = ( double * ) FLA_Obj_buffer( S );
  buf_t = ( double * ) FLA_Obj_buffer( t );

  buf_w01 = ( double * ) FLA_malloc( ( size_t ) n * sizeof( double ) );

  for ( j=0; j<min( m, n ); j++ ){
    m_min_j_min_1 = m-j-1;
    m_min_j = m-j;

    /* Update the current column by applying ( I - U S' U' ) to it
    where U and S are the parts of % these matrices that have been
    accumulated so far.
    
    w01 = - triu( S00 )' * ( trilu( A00 )' * a01 + alpha11 * a10t' + A20' * a21 );  
    
    a01     = a01     + trilu( A00 ) * w01;
    alpha11 = alpha11 + a10t         * w01;
    a21     = a21     + A20          * w01;
    */

    if ( j > 0 ){
      /* FLA_Copy( a01, w01 ); */
      FLA_C2F( dcopy )( &j, &A( 0, j ), &i_one, buf_w01, &i_one );

      /*      FLA_Trmv( FLA_LOWER_TRIANGULAR, FLA_TRANSPOSE, FLA_UNIT_DIAG, 
              A00, w01 ); */
      FLA_C2F( dtrmv )( "L", "T", "U", &j, &A( 0, 0 ), &ldim_A, buf_w01, &i_one );

      /*      FLA_Axpy_t( FLA_TRANSPOSE, alpha11, a10t, w01 ); */
      //      daxpy_( &j, &A( j, j ), &A( j, 0 ), &ldim_A, buf_w01, &i_one );

      /*      FLA_Gemv( FLA_TRANSPOSE, FLA_ONE, A20, a21, FLA_ONE, w01 );  */

      if ( m_min_j > 0 )
           FLA_C2F( dgemv )( "T", &m_min_j, &j, &d_one, &A(j, 0 ), &ldim_A, &A( j, j ), &i_one, &d_one, buf_w01, &i_one );

      /*      FLA_Trmv( FLA_UPPER_TRIANGULAR, FLA_TRANSPOSE, FLA_NONUNIT_DIAG,
              S00, w01 ); */
      FLA_C2F( dtrmv )( "U", "T", "N", &j, &S( 0, 0 ), &ldim_S, buf_w01, &i_one );

      /*      FLA_Scal( FLA_MINUS_ONE, w01 ); */
      FLA_C2F( dscal )( &j, &d_minus_one, buf_w01, &i_one );

      /*      FLA_Dot_s( FLA_ONE, a10t, w01, FLA_ONE, alpha11 ); */
      //      A( j, j ) = A( j, j ) + ddot_( &j, &A( j, 0 ), &ldim_A, buf_w01, &i_one );

      /*      FLA_Gemv( FLA_NO_TRANSPOSE, FLA_ONE, A20, w01, FLA_ONE, a21 ); */
      FLA_C2F( dgemv )( "N", &m_min_j, &j, &d_one, &A( j, 0 ), &ldim_A, buf_w01, &i_one, &d_one, &A( j, j ), &i_one );

      /*      FLA_Trmv_sx( FLA_LOWER_TRIANGULAR, FLA_NO_TRANSPOSE, FLA_UNIT_DIAG,
              FLA_ONE, A00, w01, FLA_ONE, a01 ); */
      FLA_C2F( dtrmv )( "L", "N", "U", &j, &A( 0, 0 ), &ldim_A, buf_w01, &i_one );
      FLA_C2F( daxpy )( &j, &d_one, buf_w01, &i_one, &A( 0, j ), &i_one );
    }

    /* Compute the current Householder vector 

    [ alpha11, ...
      a21, sigma11 ] = Househ2( alpha11, ...
                                a21 );
    */

    /*    FLA_Househ2( alpha11,
          a21, sigma11 );*/
    FLA_C2F( dlarfg )( &m_min_j, &A( j, j ), &A( j+1, j ), &i_one, &S( j, j ) );

    /* t( j ) = S( j, j ) */

    t( j ) = S( j, j );

    /* Update S
    s01  = - triu( S00 ) * ( a10t' + A20' * a21 ) * sigma11;
    */

    /*    FLA_Copy_t( FLA_TRANSPOSE, a10t, s01 ); */
    FLA_C2F( dcopy )( &j, &A( j, 0 ), &ldim_A, &S( 0, j ), &i_one );

    /*    FLA_Gemv( FLA_TRANSPOSE, FLA_ONE, A20, a21, FLA_ONE, s01 ); */
    FLA_C2F( dgemv )( "T", &m_min_j_min_1, &j, &d_one, &A( j+1, 0 ), &ldim_A, &A( j+1, j ), &i_one, &d_one, &S( 0, j ), &i_one );

    /*    FLA_Trmv( FLA_UPPER_TRIANGULAR, FLA_NO_TRANSPOSE, FLA_NONUNIT_DIAG,
              S00, s01 );
    */
    FLA_C2F( dtrmv )( "U", "N", "N", &j, &S( 0, 0 ), &ldim_S, &S( 0, j ), &i_one );

    /*    FLA_Scal( sigma11, s01 ); */
    FLA_C2F( dscal )( &j, &S( j, j ), &S( 0, j ), &i_one );

    /*    FLA_Scal( FLA_MINUS_ONE, s01 ); */
    FLA_C2F( dscal )( &j, &d_minus_one, &S( 0, j ), &i_one );
  }

  free( buf_w01 );

  return FLA_SUCCESS;
}



