*
*  libflame
*  An object-based infrastructure for developing high-performance
*  dense linear algebra libraries.
*
*  Copyright (C) 2008, The University of Texas
*
*  libflame is free software; you can redistribute it and/or modify
*  it under the terms of the GNU Lesser General Public License as
*  published by the Free Software Foundation; either version 2.1 of
*  the License, or (at your option) any later version.
*
*  libflame is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public
*  License along with libflame; if you did not receive a copy, see
*  http://www.gnu.org/licenses/.
*
*  For more information, please contact us at flame@cs.utexas.edu or
*  send mail to:
*
*  Field G. Van Zee and/or
*  Robert A. van de Geijn
*  The University of Texas at Austin
*  Department of Computer Sciences
*  1 University Station C0500
*  Austin TX 78712
*
      SUBROUTINE DTRTRI( UPLO, DIAG, N, A, LDA, INFO )
*
*  -- LAPACK routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     March 31, 1993
*
*     .. Scalar Arguments ..
      CHARACTER          DIAG, UPLO
      INTEGER            INFO, LDA, N
*     ..
*     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * )
*     ..
*
*  Purpose
*  =======
*
*  DTRTRI computes the inverse of a real upper or lower triangular
*  matrix A.
*
*  This is the Level 3 BLAS version of the algorithm.
*
*  Arguments
*  =========
*
*  UPLO    (input) CHARACTER*1
*          = 'U':  A is upper triangular;
*          = 'L':  A is lower triangular.
*
*  DIAG    (input) CHARACTER*1
*          = 'N':  A is non-unit triangular;
*          = 'U':  A is unit triangular.
*
*  N       (input) INTEGER
*          The order of the matrix A.  N >= 0.
*
*  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
*          On entry, the triangular matrix A.  If UPLO = 'U', the
*          leading N-by-N upper triangular part of the array A contains
*          the upper triangular matrix, and the strictly lower
*          triangular part of A is not referenced.  If UPLO = 'L', the
*          leading N-by-N lower triangular part of the array A contains
*          the lower triangular matrix, and the strictly upper
*          triangular part of A is not referenced.  If DIAG = 'U', the
*          diagonal elements of A are also not referenced and are
*          assumed to be 1.
*          On exit, the (triangular) inverse of the original matrix, in
*          the same storage format.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,N).
*
*  INFO    (output) INTEGER
*          = 0: successful exit
*          < 0: if INFO = -i, the i-th argument had an illegal value
*          > 0: if INFO = i, A(i,i) is exactly zero.  The triangular
*               matrix is singular and its inverse can not be computed.
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ZERO
      PARAMETER          ( ZERO = 0.0D+0 )
*     ..
*     .. Local Scalars ..
      LOGICAL            NOUNIT, UPPER
      INTEGER            IINFO
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ILAENV
      EXTERNAL           LSAME, ILAENV
*     ..
*     .. External Subroutines ..
      EXTERNAL           DTRMM, DTRSM, DTRTI2, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN
*     ..
*     .. Declare FLAME related stuff
      include "FLAMEf.h"
      INTEGER            A_obj( FLA_OBJ_SIZE ),
     $                   dtype,
     $                   fla_uplo, fla_diag
      PARAMETER        ( dtype = FLA_DOUBLE )
*     ..
*     .. Executable Statements ..
*
*     Initialize FLAME
*
      CALL FLA_INIT_F( IINFO )
*
*     Test the input parameters.
*
      INFO = 0
      UPPER = LSAME( UPLO, 'U' )
      NOUNIT = LSAME( DIAG, 'N' )
      IF( .NOT.UPPER .AND. .NOT.LSAME( UPLO, 'L' ) ) THEN
         INFO = -1
      ELSE IF( .NOT.NOUNIT .AND. .NOT.LSAME( DIAG, 'U' ) ) THEN
         INFO = -2
      ELSE IF( N.LT.0 ) THEN
         INFO = -3
      ELSE IF( LDA.LT.MAX( 1, N ) ) THEN
         INFO = -5
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DTRTRI', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( N.EQ.0 )
     $   RETURN
*
*     Check for singularity if non-unit.
*
      IF( NOUNIT ) THEN
         DO 10 INFO = 1, N
            IF( A( INFO, INFO ).EQ.ZERO )
     $         RETURN
   10    CONTINUE
         INFO = 0
      END IF
*
      IF( .FALSE. ) THEN
*
*        Use unblocked LAPACK code.
*
         CALL DTRTI2( UPLO, DIAG, N, A, LDA, INFO )
*
      ELSE
*
*        Use blocked FLAME code.
*
*        Preset the FLAME uplo parameter.
*
         IF( UPPER ) THEN
            fla_uplo = FLA_UPPER_TRIANGULAR
         ELSE
            fla_uplo = FLA_LOWER_TRIANGULAR
         ENDIF
*
*        Preset the FLAME diag parameter.
*
         IF( NOUNIT ) THEN
            fla_diag = FLA_NONUNIT_DIAG
         ELSE
            fla_diag = FLA_UNIT_DIAG
         ENDIF
*
*        Create base object for A_obj and attach matrix buffer.
*
         CALL FLA_OBJ_CREATE_W_OUT_BUFF_F( dtype, N, N, A_obj,
     $                                     IINFO )
         CALL FLA_OBJ_ATTACH_BUFF_F( A, LDA, A_obj, IINFO )
*
*        Compute the inverse of the triangular matrix A.
*
         CALL FLA_TRINV_F( fla_uplo, fla_diag, A_obj, IINFO )
*
*        Free A_obj base object (but leave A buffer alone).
*
         CALL FLA_OBJ_FREE_W_OUT_BUFF_F( A_obj, IINFO )
*
      END IF
*
      RETURN
*
*     End of DTRTRI
*
      END
