/*
Copyright (C) 2002 the University of Texas at Austin (U. T. Austin)

Submit software operation questions to: Robert van de Geijn,
Department of Computer Sciences,U. T. Austin, Austin, Texas 78712
(rvdg@cs.utexas.edu).

Submit commercialization requests to: Office of the Executive Vice
President and Provost, U. T. Austin, 201 Main Bldg., Austin, Texas,
78712, ATTN: Technology Licensing Specialist.
*/
#include "FLAME.h"

int FLA_LU_inc( FLA_Obj U,              
		FLA_Obj A, FLA_Obj ipiv, FLA_Obj L, int nb_alg )
{
  FLA_Obj     UTL, UTR,   U00, U01, U02,  ipivT,  ipiv0,
                          U10, U11, U12,          ipiv1,
              UBL, UBR,   U20, U21, U22,  ipivB,  ipiv2,
              AL,  AR,    A0,  A1,  A2, 
              LL,  LR,    L0,  L1,  L2,   L1_copy; 
  int b;

  FLA_Part_2x2( U,  &UTL, /**/ &UTR,  
 		 /* **************** */
		    &UBL, /**/ &UBR,  0, 0, FLA_TL );
  FLA_Part_1x2( A,  &AL,  /**/ &AR,  0, FLA_LEFT );
  FLA_Part_1x2( L,  &LL,  /**/ &LR,  0, FLA_LEFT );
  FLA_Part_2x1( ipiv, &ipivT, 
                    /* ***** */
 		      &ipivB, 0, FLA_TOP );

  while ( b = min( FLA_Obj_width( AR ), nb_alg) ){
    if ( b != nb_alg ){
      printf("b = %d, nb_alg = %d\n", b, nb_alg );
      printf("Warning: the block dimensions must be multiples of b\n");
    }

    FLA_Repart_2x2_to_3x3( UTL, /**/ UTR,       &U00, /**/ &U01, &U02, 
			/* ************* */  /* ********************* */
                                                &U10, /**/ &U11, &U12, 
			   UBL, /**/ UBR,       &U20, /**/ &U21, &U22, 
			   b, b, /* U11 split from */ FLA_BR );
    FLA_Repart_1x2_to_1x3( AL,  /**/ AR,        &A0,  /**/ &A1,  &A2,
			   b, /* A1 split from */ FLA_RIGHT );
    FLA_Repart_1x2_to_1x3( LL,  /**/ LR,        &L0,  /**/ &L1,  &L2,
			   b, /* L1 split from */ FLA_RIGHT );
    FLA_Repart_2x1_to_3x1( ipivT,                 &ipiv0,
                        /* ***** */             /* ***** */
		                                  &ipiv1,
                           ipivB,                 &ipiv2,      
                           b, /* length ipiv1 split from */ FLA_BOTTOM );

    /* ******************************************************************** */

    FLA_Copy( U11, L1 );
    FLA_Triangularize( FLA_UPPER_TRIANGULAR, FLA_NONUNIT_DIAG, L1 );

    FLA_LU_inc_panel( L1, 
		      A1,  ipiv1 );

    FLA_LU_inc_pivot_rest( U12,
			   A2, ipiv1 );

    FLA_Trsm( FLA_LEFT, FLA_LOWER_TRIANGULAR, 
	      FLA_NO_TRANSPOSE, FLA_UNIT_DIAG, 
	      ONE, L1, U12 );

    FLA_Gemm( FLA_NO_TRANSPOSE, FLA_NO_TRANSPOSE,
	      MINUS_ONE, A1, U12, ONE, A2 );

    /* Place upper triangular part of L1 back in U11 */
    /* Needs to be made more elegant! */
    FLA_Obj_create_conf_to( FLA_NO_TRANSPOSE, L1, &L1_copy );
    FLA_Copy( L1, L1_copy );
    FLA_Triangularize( FLA_UPPER_TRIANGULAR, FLA_NONUNIT_DIAG, L1_copy );
    FLA_Triangularize( FLA_LOWER_TRIANGULAR, FLA_ZERO_DIAG, U11 );
    FLA_Axpy( ONE, L1_copy, U11 );
    FLA_Obj_free( &L1_copy );

    /* ********************************************************************* */

    FLA_Cont_with_3x3_to_2x2( &UTL, /**/ &UTR,      U00, U01, /**/ U02,
			            /**/            U10, U11, /**/ U12,
                            /* ************** */ /* ****************** */
                              &UBL, /**/ &UBR,      U20, U21, /**/ U22, 
	                      /* with U11 added to submatrix */ FLA_TL );
    FLA_Cont_with_1x3_to_1x2( &AL,  /**/ &AR,        A0,  A1,  /**/ A2,
			      /* with A1 added to */ FLA_LEFT );
    FLA_Cont_with_1x3_to_1x2( &LL,  /**/ &LR,        L0,  L1,  /**/ L2,
			      /* with L1 added to */ FLA_LEFT );
    FLA_Cont_with_3x1_to_2x1( &ipivT,                  ipiv0,
		                                       ipiv1,
                            /* ***** */             /* ***** */
                              &ipivB,                  ipiv2, 
                              /* with ipiv1 added to */ FLA_TOP );
  }

  return FLA_SUCCESS;
}

