/*
Copyright (C) 2002 the University of Texas at Austin (U. T. Austin)

Submit software operation questions to: Robert van de Geijn,
Department of Computer Sciences,U. T. Austin, Austin, Texas 78712
(rvdg@cs.utexas.edu).

Submit commercialization requests to: Office of the Executive Vice
President and Provost, U. T. Austin, 201 Main Bldg., Austin, Texas,
78712, ATTN: Technology Licensing Specialist.
*/
#include "FLAME.h"

int FLA_LU_inc_apply( FLA_Obj L,               FLA_Obj BT,
		      FLA_Obj A, FLA_Obj ipiv, FLA_Obj BB, int nb_alg )
{
  FLA_Obj     AL,  AR,    A0,  A1,  A2,
              LL,  LR,    L0,  L1,  L2,  
              BTT,        BT0,              ipivT,      ipiv0,
                          BT1,              ipivB,      ipiv1,
              BTB,        BT2,                          ipiv2;
  int b;

  FLA_Part_1x2( A,  &AL,  /**/ &AR,  0, FLA_LEFT );
  FLA_Part_1x2( L,  &LL,  /**/ &LR,  0, FLA_LEFT );
  FLA_Part_2x1( BT,   &BTT, 
                    /* ***** */
 		      &BTB, 0, FLA_TOP );
  FLA_Part_2x1( ipiv, &ipivT, 
                    /* ***** */
 		      &ipivB, 0, FLA_TOP );

  while ( b = min( FLA_Obj_width( AR ), nb_alg) ){
    if ( b != nb_alg )
      printf("Warning: the block dimensions must be multiples of b\n");

    FLA_Repart_1x2_to_1x3( AL,  /**/ AR,        &A0,  /**/ &A1,  &A2,
			   b, /* A1 split from */ FLA_RIGHT );
    FLA_Repart_1x2_to_1x3( LL,  /**/ LR,        &L0,  /**/ &L1,  &L2,
			   b, /* L1 split from */ FLA_RIGHT );
    FLA_Repart_2x1_to_3x1( BTT,                   &BT0,
                        /* ***** */             /* ***** */
		                                  &BT1,
                           BTB,                   &BT2,      
                           b, /* length BT1 split from */ FLA_BOTTOM );
    FLA_Repart_2x1_to_3x1( ipivT,                 &ipiv0,
                        /* ***** */             /* ***** */
		                                  &ipiv1,
                           ipivB,                 &ipiv2,      
                           b, /* length ipiv1 split from */ FLA_BOTTOM );

    /* ******************************************************************** */

    FLA_LU_inc_pivot_rest( BT1, 
			   BB, ipiv1 );

    FLA_Trsm( FLA_LEFT, FLA_LOWER_TRIANGULAR, 
	      FLA_NO_TRANSPOSE, FLA_UNIT_DIAG, 
	      ONE, L1, BT1 );

    FLA_Gemm( FLA_NO_TRANSPOSE, FLA_NO_TRANSPOSE,
	      MINUS_ONE, A1, BT1, ONE, BB );

    /* ********************************************************************* */

    FLA_Cont_with_1x3_to_1x2( &AL,  /**/ &AR,        A0,  A1,  /**/ A2,
			      /* with A1 added to */ FLA_LEFT );
    FLA_Cont_with_1x3_to_1x2( &LL,  /**/ &LR,        L0,  L1,  /**/ L2,
			      /* with L1 added to */ FLA_LEFT );
    FLA_Cont_with_3x1_to_2x1( &BTT,                    BT0,
			                               ipiv1,
                            /* ***** */             /* ***** */
                              &BTB,                    BT2, 
                              /* with BT1 added to */ FLA_TOP );
    FLA_Cont_with_3x1_to_2x1( &ipivT,                  ipiv0,
		                                       ipiv1,
                            /* ***** */             /* ***** */
                              &ipivB,                  ipiv2, 
                              /* with ipiv1 added to */ FLA_TOP );
  }

  return FLA_SUCCESS;
}

