@echo off
@setlocal enabledelayedexpansion

rem --------------------------------------------------------------------
rem Build a dll out of a set of object files specified by the 
rem argument /objlist.  This script is inspired by and works in a similar
rem fashion to lib_to_dll.cmd, except that in this case we work on the
rem object files directly instead of extracting them from the library,
rem since it has not yet been built.
rem
rem The library name is the first argument, without the .lib extension.
rem The second argument is the path to the library, including the library
rem name, but without the extension.  This arrangement is a compromise
rem between the failings of nmake and the even grosser failings of cmd.
rem Next comes the architecture, then any possible import libraries,
rem specified by /import, and finally the object list, specified with
rem /objlist.
rem
rem The .lib file thus created is an "import" library, which one links
rem with, but the bulk of the code ends up in the associated .dll file.
rem ---------------------------------------------------------------------

set THIS_SCRIPT=%~dp0%~nx0

if "%1"=="" goto USAGE
if "%2"=="" goto USAGE
if "%3"=="" goto USAGE

set gd_lib_name=%1
set gd_link=%gd_lib_name%-static.link
set gd_def=%gd_lib_name%-static.def

rem are we being called as a "subroutine"?
if not "%2"=="/addobjects" goto CHECK_REMAINING_ARGS
rem "subroutine" call
:ADDOBJ
set gd_obj=%3
if not defined gd_obj goto END
echo %gd_obj% >>%gd_link%
set gd_syms=%gd_obj%.syms
dumpbin /symbols %gd_obj% | findstr /c:"SECT2  notype ()    External" > %gd_syms%
for /f "tokens=8" %%i in (%gd_syms%) do (
    rem write symbol to def file, after removing initial underscore, if any
    set gd_sym1=%%i
    set gd_sym2=!gd_sym1:~0,1!
    set gd_sym3=!gd_sym1:~1!
    if "!gd_sym2!"=="_" (set gd_sym3=!gd_sym1:~1!) else (set gd_sym3=!gd_sym1!)
    echo !gd_sym3! >> %gd_def%
)
del /f /q %gd_syms%
shift /3
goto ADDOBJ


:CHECK_REMAINING_ARGS
rem check remaining args
if "%4"=="" goto USAGE
set LIBPATHSFILE=%3
goto PARSE_ARGS

:PARSE_ARGS
set IMPORT=
set OBJLIST=
:ARGLOOP
if "%4"=="" goto ENDARGLOOP
if /i not "%4"=="/import" goto OBJARG
set IMPORT=!IMPORT! %5
goto SHIFT
:OBJARG
if /i not "%4"=="/objlist" goto ENDARGLOOP
set OBJLIST=%5
:SHIFT
shift /4
shift /4
goto ARGLOOP
:ENDARGLOOP

if defined OBJLIST goto COMPILER_SETUP
echo Error: must supply /objlist <file with list of object names>
goto USAGE

:COMPILER_SETUP
set gd_path=%2
set gd_dll_path=%gd_path%.dll
set gd_main_c=dll_main__%gd_lib_name%.c
set gd_main_obj=dll_main__%gd_lib_name%.obj

rem start definition file
echo EXPORTS >%gd_def%

rem create C file for dll_main
for /F "tokens=*" %%i in ("#include <windows.h>") do echo %%i >%gd_main_c%
echo. >>%gd_main_c%
echo BOOLEAN WINAPI DllMain( >>%gd_main_c%
echo 	HINSTANCE hDllHandle, >>%gd_main_c%
echo 	DWORD     nReason,    >>%gd_main_c%
echo 	LPVOID    Reserved){  >>%gd_main_c%
echo.                        >>%gd_main_c%
echo BOOLEAN bSuccess = TRUE;>>%gd_main_c%
echo.                        >>%gd_main_c%
echo	switch (nReason){     >>%gd_main_c%
echo		case DLL_PROCESS_ATTACH: >>%gd_main_c%
echo			DisableThreadLibraryCalls( hDllHandle ); >>%gd_main_c%
echo		break; >>%gd_main_c%
echo		case DLL_PROCESS_DETACH: >>%gd_main_c%
echo		break; >>%gd_main_c%
echo.            >>%gd_main_c%
echo	}; >>%gd_main_c%
echo.   >>%gd_main_c%
echo	return bSuccess; >>%gd_main_c%
echo }; >>%gd_main_c%
echo.>>%gd_main_c%

rem set up link file
echo /nologo /LD /Fe%gd_dll_path% /Zp8 /MD /GF /D_WIN32_WINNT=0x0500 /DWINVER=0x0500 /D_WIN32_IE=0x0501 /DWIN32_LEAN_AND_MEAN=1 /DSECURITY_WIN32=1 /D_CRT_SECURE_NO_DEPRECATE=1 /D_CRT_NONSTDC_NO_DEPRECATE=1 > %gd_link%
echo %gd_main_obj% >>%gd_link%

for /f "tokens=* delims=" %%i in (%OBJLIST%) do call %THIS_SCRIPT% %gd_lib_name% /addobjects %%i

rem add standard import libraries
echo user32.lib wininet.lib advapi32.lib ole32.lib oleaut32.lib shell32.lib iphlpapi.lib odbc32.lib winmm.lib vfw32.lib gdi32.lib activeds.lib adsiid.lib secur32.lib >>%gd_link%

rem add command-line import libraries
if defined IMPORT echo !IMPORT! >>%gd_link%

rem add export specification
echo %gd_def% >>%gd_link%

rem add link spec
echo /link /LIBPATH:. /nodefaultlib:MSVCRTD /nodefaultlib:MSVCR90 /nodefaultlib:MSVCRT >>%gd_link%
type %LIBPATHSFILE% >>%gd_link%

rem create dll, import lib and export file
cl /nologo /c /O2 /Fo%gd_main_obj% %gd_main_c% >> gendll-cl.log
ifort @%gd_link%

:CLEANUP
del /F /Q %gd_link% %gd_def% %gd_main_c% %gd_main_obj% gendll-cl.log
goto END


:USAGE
echo. 
echo. gendll.cmd
echo. 
echo. Generate a dynamically-linked library from a set of object files
echo. specified in objlist_file.
echo. 
echo. Usage:
echo.   %0 dllname dllpath libpaths_file {/import importlib} /objlist objlist_file
echo.
echo.     dllname       -- the name of the DLL being created, with no extension.
echo.     dllpath       -- the path to the DLL being created, with no extension.
echo.     libpaths_file -- the path to a file containing a list of library paths
echo.                      that may be needed to successfully link the DLL being
echo                       created.
echo.     importlib     -- the path to a .lib library that you wish to import into
echo.                      the DLL being created.
echo.     objlist_file  -- the path to a file containing the list of object files
echo.                      that make up the bulk of the DLL being created.
echo.

:END
endlocal
